@@ -5,7 +5,7 @@
 #' 
 #' @param cex Multiplier for font size
 #' @export
-latex2exp_examples <- function(cex=1) {
+latex2exp_examples <- function(cex = 1) {
   oldpar <- par(no.readonly = TRUE)
   on.exit(suppressWarnings(par(oldpar)))
   
@@ -31,9 +31,8 @@ latex2exp_examples <- function(cex=1) {
   x <- 0
   y <- seq(0.95, 0.05, length.out = length(examples))
   
-  text(
-    0.5, y, str_c("TeX(r\"(", examples, ")\")"), pos = 2, cex = 0.5 * cex, family = 'mono'
-  )
-  text(0.5, y, TeX(examples), pos = 4, cex=cex)
+  text(0.5, y, paste0("TeX(r\"(", examples, ")\")"), pos = 2, cex = 0.5 * cex,
+    family = 'mono')
+  text(0.5, y, TeX(examples), pos = 4, cex = cex)
   return(invisible(TRUE))
-}
\ No newline at end of file
+}
